<?php

namespace App\Http\Controllers\Admin;

use App\Models\ProjectUser;
use App\Models\User;
use App\Models\UserAuth;
use Illuminate\Http\Request;

class UserAuthController extends BaseController
{
    protected $model;

    protected $department;

    protected $model_name = '用户授权';

    protected $pre_uri = '/admin/UserAuth/';

    protected $view_path = 'admin.user-auth.';

    protected $redirect_index = '/admin/UserAuth/index';

    public function __construct()
    {
        parent::__construct();
        $this->model = new UserAuth();
    }

    public function index()
    {
        list($model, $model_name, $pre_uri) = array($this->model, $this->model_name, $this->pre_uri);
        return view($this->view_path . 'index', compact('model', 'model_name','pre_uri'));
    }

    public function get(Request $request)
    {
        $items = $this->model->orderBy('id', 'desc');

        $tmp_items = collect(['name']);
        foreach($tmp_items as $tmp_item) {
            if($request->has($tmp_item) && !empty($request->input($tmp_item))) {
                $items = $items->where($tmp_item, 'like', '%' . $request->input($tmp_item) . '%');
            }
        }

        $select_items = collect([]);
        foreach($select_items as $select_item) {
            if($request->has($select_item) && !empty($request->input($select_item))) {
                $items = $items->where($select_item, '=', $request->input($select_item));
            }
        }

        $items = $items->paginate();

        foreach($items as $item) {
            $item->project_name = $item->project ? $item->project->name : '';
            $item->project_role_name = $item->project_role ? $item->project_role->name : '';
            $item->active_label = $item->getActive('label');
            $item->avatar = $item->user ? $item->user->getAvatar() : '';
        }

        return response()->json(['code' => 0, 'message' => '', 'count' => $items->total(), 'data' => $items->items()]);
    }

    public function change(Request $request)
    {
        $item = $this->model->find($request->input('id'));
        if(!$item) return response()->json(['status' => 'fail', 'info' => '找不到数据']);
        if($request->input('active') == 1) {
            ProjectUser::firstOrCreate([
                'project_id' => $item['project_id'],
                'user_id' => $item['user_id'],
                'project_role_id' => $item['project_role_id']
            ]);
            User::where('id', $item['user_id'])->update(['name' => $item['name'], 'password' => bcrypt(123456)]);
            $item->update(['active' => 1]);
        }
        return response()->json(['status' => 'success', 'info' => '操作成功']);
    }

    public function create()
    {
        list($model, $model_name, $pre_uri) = array($this->model, $this->model_name, $this->pre_uri);
        return view($this->view_path . 'create', compact('model', 'model_name','pre_uri'));
    }

    public function store(Request $request)
    {
        if(empty($request->input('data')) || !is_array($request->input('data'))) return back()->withErrors(['sg_error_info' => '数据错误']);
        $validator = $this->model->getValidator($request, 'store');
        if($validator->fails()) {
            return back()->withErrors($validator)->withInput();
        }
        $data = $request->input('data');
        $res = $this->model->create($data);
        if(empty($res)) return back()->withErrors(['sg_error_info' => '保存失败']);
        return redirect($this->pre_uri . 'create')->with(['sg_success_info' => '创建成功']);
    }

    public function edit(Request $request)
    {
        if(empty($request->input('id')) || empty($item = $this->model->find($request->input('id')))) return back()->withErrors(['sg_error_info' => '找不到要编辑的数据']);
        list($model, $model_name, $pre_uri) = array($this->model, $this->model_name, $this->pre_uri);
        return view($this->view_path . 'edit', compact('model', 'model_name', 'pre_uri', 'item'));
    }

    public function update(Request $request)
    {
        if(empty($request->input('id')) || empty($item = $this->model->find($request->input('id')))) return back()->withErrors(['sg_error_info' => '找不到要编辑的数据']);
        if(empty($request->input('data')) || !is_array($request->input('data'))) return back()->withErrors(['sg_error_info' => '数据错误']);
        $validator = $this->model->getValidator($request, 'update');
        if($validator->fails()) {
            return back()->withErrors($validator)->withInput();
        }
        $data = $request->input('data');
        $res = $this->model->where('id', $request->input('id'))->update($data);
        if(!$res) return back()->withErrors(['sg_error_info' => '数据库保存失败！']);
        return back()->with(['sg_success_info' => '编辑成功']);
    }

    public function delete(Request $request)
    {
        if(empty($request->input('id')) || empty($item = $this->model->find($request->input('id')))) return response()->json(['status' => 'fail', 'info' => '找不到要删除的数据']);
        $res = $item->delete();
        if (!$res) return response()->json(['status' => 'fail', 'info' => '删除失败']);
        return response()->json(['status' => 'success', 'info' => '操作成功']);
    }
}
